<?php

namespace PortlandLabs\DocumentLibrary\FilterableList;

use Closure;
use Concrete\Core\Attribute\AttributeKeyInterface;
use Concrete\Core\Attribute\Key\Key;
use Concrete\Core\File\FolderItemList;
use Concrete\Core\Search\ItemList\Database\AttributedItemList;
use Concrete\Core\Search\StickyRequest;
use Concrete\Core\Tree\Node\Type\FileFolder;

/**
 * Overrides all methods to pass through to the parent list
 * provides the filterByAttribute method
 */
class AttributeFilterableList extends AttributedItemList
{

    protected $parentList;

    public function __construct(FolderItemList $parentList)
    {
        $this->parentList = $parentList;
    }

    protected function getAttributeKeyClassName()
    {
        return '\\Concrete\\Core\\Attribute\\Key\\FileKey';
    }

    protected function createPaginationObject()
    {
        return $this->parentList->getPagination();
    }

    public function setPermissionsChecker(Closure $checker)
    {
        $this->parentList->setPermissionsChecker($checker);
    }

    public function ignorePermissions()
    {
        $this->parentList->ignorePermissions();
    }

    public function createQuery()
    {
        $this->parentList->createQuery();
    }

    public function getTotalResults()
    {
        return $this->parentList->getTotalResults();
    }

    public function getResult($queryRow)
    {
        return $this->parentList->getResult($queryRow);
    }

    public function checkPermissions($mixed)
    {
        return $this->parentList->checkPermissions($mixed);
    }

    public function filterByParentFolder(FileFolder $folder)
    {
        $this->parentList->filterByParentFolder($folder);
    }

    public function filterByType($type)
    {
        $this->parentList->filterByType($type);
    }

    public function deliverQueryObject()
    {
        return $this->parentList->deliverQueryObject();
    }

    public function sortByNodeName()
    {
        $this->parentList->sortByNodeName();
    }

    public function sortByNodeType()
    {
        $this->parentList->sortByNodeType();
    }

    public function finalizeQuery(\Doctrine\DBAL\Query\QueryBuilder $query)
    {
        return $this->parentList->finalizeQuery($query);
    }

    public function getQueryObject()
    {
        return $this->parentList->getQueryObject();
    }

    public function executeGetResults()
    {
        return $this->parentList->executeGetResults();
    }

    public function debugStart()
    {
        $this->parentList->debugStart();
    }

    public function debugStop()
    {
        $this->parentList->debugStop();
    }

    public function filter($field, $value, $comparison = '=')
    {
        $this->parentList->filter($field, $value, $comparison);
    }

    public function debug()
    {
        $this->parentList->debug();
    }

    public function isDebugged()
    {
        return $this->parentList->isDebugged();
    }

    public function sortBy($field, $direction = 'asc')
    {
        $this->parentList->sortBy($field, $direction);
    }

    public function sanitizedSortBy($field, $direction = 'asc')
    {
        $this->parentList->sanitizedSortBy($field, $direction);
    }

    public function getResults()
    {
        return $this->parentList->getResults();
    }

    public function getActiveSortColumn()
    {
        return $this->parentList->getActiveSortColumn();
    }

    public function isActiveSortColumn($field)
    {
        return $this->parentList->isActiveSortColumn($field);
    }

    public function disableAutomaticSorting()
    {
        $this->parentList->disableAutomaticSorting();
    }

    public function getSortClassName($column)
    {
        return $this->parentList->getSortClassName($column);
    }

    public function getSortURL($column, $dir = 'asc', $url = false)
    {
        return $this->parentList->getSortURL($column, $dir, $url);
    }

    public function getActiveSortDirection()
    {
        return $this->parentList->getActiveSortDirection();
    }

    public function getQuerySortColumnParameter()
    {
        return $this->parentList->getQuerySortColumnParameter();
    }

    public function getQueryPaginationPageParameter()
    {
        return $this->parentList->getQueryPaginationPageParameter();
    }

    public function getQuerySortDirectionParameter()
    {
        return $this->parentList->getQuerySortDirectionParameter();
    }

    public function setItemsPerPage($itemsPerPage)
    {
        $this->parentList->setItemsPerPage($itemsPerPage);
    }

    public function getPagination()
    {
        return $this->parentList->getPagination();
    }

    public function setupAutomaticSorting(StickyRequest $request = null)
    {
        $this->parentList->setupAutomaticSorting($request);
    }

    public function get()
    {
        return $this->parentList->get();
    }

    /**
     * Filters by a attribute.
     */
    public function filterByAttribute($handle, $value, $comparison = '=')
    {
        $keyClass = $this->getAttributeKeyClassName();
        /** @var \Concrete\Core\Entity\Attribute\Key\FileKey $key */
        $key = $keyClass::getByHandle($handle);

        if (!is_object($key)) {
            throw new \Exception(t('Unable to find attribute %s', $handle));
        }
        $key->getController()->filterByAttribute($this, $value, $comparison);
    }

}
